@@ -11,10 +11,11 @@
 #' @param uniform logical indicating whether to create the grid on \code{vars} uniformly or to sample without replacement from the empirical distribution of those \code{vars}.
 #' @param points a named list which gives specific points for \code{vars}. specifying this argument overrides \code{uniform}.
 #' @param int.points a integer vector giving indices of the points in \code{data} to marginalize over.
-#' @param aggregate.fun what function to aggregate the predictions with. this function takes a single argument \code{x} and returns a vector. the default is \code{sum(x) / length(x)}.
+#' @param aggregate.fun what function to aggregate the predictions with. this function takes a single argument \code{x} and returns a vector. the default is \code{sum(x) / length(x)}. If \code{weight.fun} is used, this function must also take a numeric parameter \code{w}.
 #' @param predict.fun what function to generate predictions using \code{model}. default is the predict method for \code{model}. this function must have two arguments, \code{object} and \code{newdata}.
+#' @param weight.fun a function to construct weights for \code{aggregate.fun}. this allows Monte-Carlo integration on a grid without assuming a uniform distribution for said grid. the function should take two arguments, \code{design} and \code{data}, both of which are \code{data.frame}s of the same column (but different row) dimension, and should return a numeric vector of the same length as the number of rows in \code{design}. If this argument is used \code{aggregate.fun} must also have an argument \code{w} which is the result of \code{weight.fun}.
 #'
-#' @return a named list with an element "prediction" which contains an array, matrix, or vector of dimension \code{n[1]}, the column dimension of the output of \code{predict.fun}, and the dimension of the output from \code{aggregate.fun}.
+#' @return a \code{data.table} with columns for predictions and \code{vars}.
 #'
 #' @examples
 #' X = replicate(3, rnorm(100))
@@ -26,32 +27,24 @@
 #'   aggregate.fun = function(x) c("mean" = mean(x), "variance" = var(x)))
 #' @export
 marginalPrediction = function(data, vars, n, model, uniform = TRUE, points, int.points,
- aggregate.fun = function(x) sum(x) / length(x), predict.fun = function(object, newdata)
-   predict(object, newdata = newdata)) {
+  aggregate.fun = function(x) sum(x) / length(x),
+  predict.fun = function(object, newdata) predict(object, newdata = newdata),
+  weight.fun = NULL) {
 
-  assertFunction(aggregate.fun, args = "x", nargs = 1L)
-  assertFunction(predict.fun, args = c("object", "newdata"), nargs = 2L)
+  assertFunction(aggregate.fun, args = "x")
+  assertFunction(predict.fun, args = c("object", "newdata"))
 
   design = makeDesign(data, vars, n, uniform, points, int.points)
-  n[1] = nrow(unique(design[, vars, drop = FALSE])) ## improve
   preds = predict.fun(model, design)
-    
-  if (is.matrix(preds) | is.data.frame(preds)) {
-    save = colnames(preds)
-    preds = array(preds, c(n, ncol(preds)))
-    mp = apply(preds, c(1, 3), aggregate.fun)
-    colnames(mp) = save
-  } else {
-    preds = array(preds, n)
-    mp = apply(preds, 1, aggregate.fun)
-    if (!is.vector(mp)) {
-      ## apply with a vector valued fun needs a transpose
-      mp = t(mp)
-    }
-  }
 
-  list(
-    "prediction" = mp,
-    "points" = unique(design[, vars, drop = FALSE]) ## improve
-  )
+  if (!is.null(weight.fun)) {
+    assertFunction(weight.fun, args = c("design", "data"))
+    assertFunction(aggregate.fun, args = c("x", "w"))
+    w = weight.fun(design, data)
+    mp = data.table(w, preds, design[, vars, drop = FALSE], key = vars)
+    mp[, as.list(unlist(lapply(.SD[, !"w", with = FALSE], aggregate.fun, w = w))), by = vars]
+  } else {
+    mp = data.table(preds, design[, vars, drop = FALSE], key = vars)
+    mp[, as.list(unlist(lapply(.SD, aggregate.fun))), by = vars]
+  }
 }
